/*
 AlarmSM 433MHz Remote Control
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 22/12/2014
 Last Modified: 26/12/2014
 
 Transmit the remote control request when the button is pressed.
 
 */

// ATtiny45/85 Pin map
//
//                          +-\/-+
//              Reset PB5  1|o   |8  VCC
//   System on Button PB3  2|    |7  PB2 System test Button
//  System off Button PB4  3|    |6  PB1 LED
//                    GND  4|    |5  PB0 433MHz Out
//                          +----+

#define F_CPU 1000000 // 1 MHz

#define RF_OUT PB0
#define LED PB1

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <string.h>
#include "setup.c"
#include "hmac-sha1/hmac-sha1.h"

// 256bit random number for rolling code (you should change this when re-programming, each device should be different)
uint8_t random_number[] = {0x27, 0x9B, 0x6D, 0xD8, 0xBA, 0x17, 0x62, 0xD7,
									0xBB, 0x14, 0xF7, 0xDF, 0xDA, 0x41, 0xEE, 0x66,
									0x8A, 0x7A, 0x8D, 0X6F, 0xFA, 0x4F, 0xDA, 0xFC,
									0x41, 0x01, 0x12, 0x3B, 0x9F, 0xF7, 0xF2, 0xAF};

int main(void) {
	setup();
	
	while(1) {
		// Sleep
		sbi(GIMSK,PCIE); // Enable pin change interrupt
		system_sleep();
		cbi(GIMSK,PCIE); // Disable pin change interrupt
		
		_delay_ms(10); // Wait for button to settle
		
		// Read button state
		remote_control_state = NO_INPUT;
		uint8_t waitForInputCounter = 0;
		while (remote_control_state == NO_INPUT && waitForInputCounter < 40) {
			if (PINB & (1<<PB3)) {
				remote_control_state = SYSTEM_ON;
			}
			else if (PINB & (1<<PB4)) {
				remote_control_state = SYSTEM_OFF;
			}
			else if (PINB & (1<<PB2)) {
				remote_control_state = SYSTEM_TEST;
			}
			_delay_ms(5);
			waitForInputCounter++;
		}
		
		if (remote_control_state != NO_INPUT) {
			// Increment our 256bit random number
			random_number[31]++;
			if (random_number[31] == 0) {
				for (int8_t a = 31; a >= 0; a--) {
					if (random_number[a] == 0) {
						random_number[(a-1)]++;
					}
					else {
						break;
					}
				}
			}
			
			// Generate the random block (160 bit) using SHA1 from our random number
			sha1(data_out, random_number, 256);
			
			// Set as RF output
			DDRB |= (1<<RF_OUT);
			
			// Transmit as long as the button is held down
			PORTB |= (1<<LED);
			while ((PINB & (1<<PB3)) | (PINB & (1<<PB2)) | (PINB & (1<<PB4))) {
				for (uint8_t x = 0; x < 10; x++) {
					rf433_transmit_data();
					_delay_ms(5);
				}
			}
			PORTB &= ~(1<<LED);
			
			// Set back as input
			rf_433_low;
			DDRB &= ~(1<<RF_OUT);
		}
		
		_delay_ms(250); // Wait a little bit for button to settle
	}
}

ISR(PCINT0_vect) { }
