/*
 AlarmSM 433MHz Remote Control
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 22/12/2014
 Last Modified: 26/12/2014
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define NO_INPUT 0
#define SYSTEM_ON 240
#define SYSTEM_OFF 15
#define SYSTEM_TEST 170

#define rf_433_high	PORTB |=  (1<<RF_OUT);
#define rf_433_low	PORTB &= ~(1<<RF_OUT);
#define RF_BIT1_DELAY 400
#define RF_BIT0_DELAY 200
#define RF_BLANK_DELAY 200
#define RF_BIT_TO_CHECKSUM_DELAY 50

uint8_t data_out[21];
uint8_t remote_control_state = NO_INPUT;

// Send data
void rf433_transmit_data(void) {
	// Blanking period
	rf_433_low;
	_delay_ms(1);
	
	// Copy in remote control state
	data_out[20] = remote_control_state;
	
	// Transmit data out
	for (int x = 0; x < 21; x++) {
		uint8_t checksum = 0;
		for (uint8_t i = 0; i < 8; i++) {
			if (data_out[x] & (1<<i)) { // High bit
				rf_433_high;
				_delay_us(RF_BIT1_DELAY);
				rf_433_low;
				_delay_us(RF_BLANK_DELAY);
				checksum ^= 1;
				
			}
			else { // Low bit
				rf_433_high;
				_delay_us(RF_BIT0_DELAY);
				rf_433_low;
				_delay_us(RF_BLANK_DELAY);
				checksum ^= 0;
			}
		}
		
		// Transmit checksum
		if (checksum == 1) {
			rf_433_high;
			_delay_us(RF_BIT1_DELAY);
			rf_433_low;
			_delay_us(RF_BLANK_DELAY);
			
		} else {
			rf_433_high;
			_delay_us(RF_BIT0_DELAY);
			rf_433_low;
			_delay_us(RF_BLANK_DELAY);
		}
	}
	
	rf_433_low;
}

// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Setup the ATtiny
void setup(void) {
	// Pin interrupt setup for buttons
	sbi(GIMSK,PCIE);
	sbi(PCMSK,PCINT2);
	sbi(PCMSK,PCINT3);
	sbi(PCMSK,PCINT4);
	
	// Set outputs
	DDRB |= (1<<LED);
	
	sei(); // Turn on interrupts
}
