/*
 AlarmSM 433MHz Remote Control Server
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 22/12/2014
 Last Modified: 26/12/2014
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define NO_INPUT 0
#define SYSTEM_ON 240
#define SYSTEM_OFF 15
#define SYSTEM_TEST 170

#define RAND_R1_ADDITION_LOCATION 2
#define RAND_R2_ADDITION_LOCATION 6

#define R1_SET_LOCATION 20
#define R2_SET_LOCATION 21
#define BLANK_EEPROM_BYTE 255
volatile uint8_t R1Set = BLANK_EEPROM_BYTE;
volatile uint8_t R2Set = BLANK_EEPROM_BYTE;

uint8_t data_in[21];
uint8_t random_number_sha[20];
uint8_t remote_control_state = NO_INPUT;

extern uint8_t random_number_r1[32];
extern const uint8_t random_number_r1_base[];
extern uint32_t random_number_r1_addition;
extern uint8_t random_number_r2[32];
extern const uint8_t random_number_r2_base[];
extern uint32_t random_number_r2_addition;

// Initialise the remote control rolling code from the base number and increment as many times as needed
void rc_code_init(uint8_t *random_number_remote, const uint8_t *random_number_remote_base, uint32_t rand_end) {
	for (uint8_t x = 0; x < 32; x++) {
		random_number_remote[x] = pgm_read_byte(&random_number_remote_base[x]);
	}
	for (uint32_t x = 0; x < rand_end; x++) {
		random_number_remote[31]++;
		if (random_number_remote[31] == 0) {
			for (int8_t a = 31; a >= 0; a--) {
				if (random_number_remote[a] == 0) {
					random_number_remote[(a-1)]++;
				}
				else {
					break;
				}
			}
		}
	}
}

// Receive data
uint8_t rf433_receive_data(void) {
	// Clear data in
	for (int x = 0; x < 21; x++) {
		data_in[x] = 0;
	}
	
	// Wait for blanking of 500uS
	uint8_t blanking = 0;
	while (blanking <= 50) {
		if (!(PINB & (1<<RF_IN))) {
			blanking++;
		}
		else {
			blanking = 0;
		}
		_delay_us(5);
	}
	
	// Wait for first high transition
	while (!(PINB & (1<<RF_IN)));
	
	// Read data
	for (int x = 0; x < 21; x++) {
		uint8_t bitCounter = 0;
		uint8_t checksum = 0;
		while (bitCounter < 9) {
			// Wait for high transition
			uint8_t timeCounter = 0;
			while (!(PINB & (1<<RF_IN))) {
				_delay_us(1);
				timeCounter++;
				if (timeCounter >= 100) {
					return 0;
				}
			}
			
			// Start timer to check how long it's high for
			timeCounter = 0;
			for (timeCounter = 0; timeCounter <= 66; timeCounter++) {
				_delay_us(1);
				if (!(PINB & (1<<RF_IN))) { // Break when it goes low
					break;
				}
			}
			if (timeCounter >= 66) { // High for too long, exit
				return 0;
			}
			
			// Process counter
			if (bitCounter < 8) {
				if (timeCounter >= 10 && timeCounter <= 35) { // 0 bit
					checksum ^= 0;
				}
				else if (timeCounter >= 36 && timeCounter <= 65) { // 1 bit
					data_in[x] |= (1<<bitCounter);
					checksum ^= 1;
				}
			}
			else { // Check the checksum
				if (timeCounter >= 10 && timeCounter <= 35 && checksum != 0) {
					return 0;
				}
				else if (timeCounter >= 36 && timeCounter <= 65 && checksum != 1) {
					return 0;
				}
			}
			bitCounter++;
		}
	}
	
	// Update remote control state
	remote_control_state = data_in[20];
	
	return 1;
}

// Setup the ATtiny
void setup(void) {
	// Set outputs
	DDRB |= (1<<LED) | (1<<SYSTEM_STATE);
	
	sei(); // Turn on interrupts
	
	// Check if this is the first boot of the MCU after programming, if it isn't then read the rolling code addition from EEPROM
	R1Set = eeprom_read_byte((uint8_t*) R1_SET_LOCATION);
	if (R1Set != BLANK_EEPROM_BYTE) {
		random_number_r1_addition = (uint32_t) eeprom_read_byte((uint8_t*) RAND_R1_ADDITION_LOCATION) << 24 | (uint32_t) eeprom_read_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+1) << 16 | (uint32_t) eeprom_read_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+2) << 8 | (uint32_t) eeprom_read_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+3);
		rc_code_init(random_number_r1, random_number_r1_base, random_number_r1_addition);
	}
	else {
		rc_code_init(random_number_r1, random_number_r1_base, 0);
	}
	
	R2Set = eeprom_read_byte((uint8_t*) R2_SET_LOCATION);
	if (R2Set != BLANK_EEPROM_BYTE) {
		random_number_r2_addition = (uint32_t) eeprom_read_byte((uint8_t*) RAND_R2_ADDITION_LOCATION) << 24 | (uint32_t) eeprom_read_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+1) << 16 | (uint32_t) eeprom_read_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+2) << 8 | (uint32_t) eeprom_read_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+3);
		rc_code_init(random_number_r2, random_number_r2_base, random_number_r2_addition);
	}
	else {
		rc_code_init(random_number_r2, random_number_r2_base, 0);
	}
}
